/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.collaboration.request.CollaborationRequestServiceGrpc.CollaborationRequestServiceBlockingStub;
import de.ozgcloud.collaboration.request.GrpcFindRequestsRequest;
import de.ozgcloud.collaboration.request.GrpcGetRequestRequest;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
public class CollaborationRemoteService {

	@GrpcClient(GrpcUtil.COLLABORATION_MANAGER_GRPC_CLIENT)
	private CollaborationRequestServiceBlockingStub serviceStub;

	@Autowired
	private CollaborationMapper mapper;

	public Collaboration getById(String collaborationId) {
		return mapper.fromCollaborationRequest(serviceStub.getRequest(buildGetRequest(collaborationId)).getRequest());
	}

	public Stream<Collaboration> getCollaborations(String vorgangId) {
		return serviceStub.findRequests(buildSearchRequest(vorgangId))
				.getRequestsList().stream()
				.map(mapper::fromCollaborationRequest);
	}

	GrpcGetRequestRequest buildGetRequest(String collaborationId) {
		return GrpcGetRequestRequest.newBuilder().setId(collaborationId).build();
	}

	GrpcFindRequestsRequest buildSearchRequest(String vorgangId) {
		return GrpcFindRequestsRequest.newBuilder().setVorgangId(vorgangId).build();
	}
}
