/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.forwarding;

import java.util.List;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.alfa.forwarding.Forwarding.Status;

@RestController
@RequestMapping(ForwardingController.LIST_PATH)
public class ForwardingController {

	static final String LIST_PATH = "/api/forwardings"; // NOSONAR

	static final String PARAM_VORGANG_ID = "vorgangId";

	static final Predicate<Forwarding> IS_FAILED_FORWARDING = forwarding -> forwarding.getStatus() == Status.FAILED
			|| forwarding.getStatus() == Status.SEND_ERROR;

	@Autowired
	private ForwardingService service;
	@Autowired
	private ForwardingModelAssembler modelAssembler;

	@GetMapping(params = { PARAM_VORGANG_ID })
	public CollectionModel<EntityModel<Forwarding>> findByVorgangId(@RequestParam String vorgangId) {
		return modelAssembler.toCollectionModel(findByVorgang(vorgangId));
	}

	public List<Forwarding> findFailedForwardings(String vorgangId) {
		return findByVorgang(vorgangId).filter(IS_FAILED_FORWARDING).toList();
	}

	public Stream<Forwarding> findByVorgang(String vorgangId) {
		return service.findByVorgang(vorgangId);
	}
}