/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.historie;

import java.util.List;
import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandService;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
class HistorieService {

	private final CommandService commandService;
	private final List<HistorieProcessor> processors;

	public Stream<Command> findFinishedCommands(String vorgangId) {
		var commands = commandService.findFinishedCommands(vorgangId);
		return processCommands(commands);
	}

	Stream<Command> processCommands(Stream<Command> commands) {
		for (var processor : processors) {
			commands = processor.process(commands);
		}
		return commands;
	}
}