package de.ozgcloud.alfa.kommentar;

import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.collections.MapUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandBodyMapper;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.alfa.historie.HistorieProcessor;

@Component
class KommentarHistorieProcessor implements HistorieProcessor {

	@Override
	public Stream<Command> process(Stream<Command> commands) {
		return commands.map(this::translateCreateKommentar).map(this::translateEditKommentar);
	}

	Command translateCreateKommentar(Command command) {
		if (isCreateAttachedItemCommand(command) && hasKommentarItemName(command)) {
			return command.toBuilder().order(LegacyOrder.CREATE_KOMMENTAR).build();
		}
		return command;
	}

	private boolean isCreateAttachedItemCommand(Command command) {
		return CommandOrder.CREATE_ATTACHED_ITEM == command.getCommandOrder();
	}

	Command translateEditKommentar(Command command) {
		if (isUpdateAttachedItemCommand(command) && hasKommentarItemName(command)) {
			return command.toBuilder().order(LegacyOrder.EDIT_KOMMENTAR).build();
		}
		return command;
	}

	private boolean isUpdateAttachedItemCommand(Command command) {
		return CommandOrder.UPDATE_ATTACHED_ITEM == command.getCommandOrder();
	}

	private boolean hasKommentarItemName(Command command) {
		return getItemName(command).filter(itemName -> Kommentar.class.getSimpleName().equals(itemName)).isPresent();
	}

	private Optional<String> getItemName(Command command) {
		return Optional.ofNullable(MapUtils.getString(command.getBody(), CommandBodyMapper.ITEM_NAME_PROPERTY));
	}

}
