/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang;

import java.util.List;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;

import de.ozgcloud.alfa.common.BaseTypesMapper;
import de.ozgcloud.alfa.common.clientattribute.ClientAttribute;
import de.ozgcloud.alfa.common.clientattribute.ClientAttributeUtils;
import de.ozgcloud.alfa.common.user.UserIdMapper;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttribute;
import de.ozgcloud.vorgang.grpc.clientAttribute.GrpcClientAttributeValue;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangHead;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangWithEingang;

@Mapper(uses = { EingangMapper.class, UserIdMapper.class, BaseTypesMapper.class, ServiceKontoMapper.class })
interface VorgangWithEingangMapper {

	@Mapping(target = "hasNewPostfachNachricht", source = "clientAttributesList")
	VorgangWithEingang toVorgangWithEingang(GrpcVorgangWithEingang vorgangMitEingang);

	default boolean mapHasNewPostfachNachricht(List<GrpcClientAttribute> clientAttributes) {
		return ClientAttributeUtils
				.findGrpcValue(ClientAttribute.HAS_NEW_POSTFACH_NACHRICHT, clientAttributes)
				.map(GrpcClientAttributeValue::getBoolValue)
				.orElse(false);
	}

	@Mapping(target = "locked", expression = "java(vorgangHead.hasLock())")
	VorgangHead toVorgangHead(GrpcVorgangHead vorgangHead);
}