/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.downloadtoken;

import static de.ozgcloud.alfa.JwtTokenUtil.*;

import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import io.jsonwebtoken.JwtBuilder;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;

public class DownloadTokenTestFactory {
	static final String TYP = "typ";
	static final String SUBJECT = "subject";
	static final String FIRSTNAME_CLAIM_VALUE = UserProfileTestFactory.FIRSTNAME;
	static final String LASTNAME_CLAIM_VALUE = UserProfileTestFactory.LASTNAME;
	final static Collection<String> ORGE_IDS = List.of("258994");
	static final List<?> ROLE_CLAIM_VALUE = List.of();
	static final long VALIDITY = 5000;
	static final Map<String, Object> CLAIMS = new HashMap<>(Map.of(
			FIRSTNAME_CLAIM, FIRSTNAME_CLAIM_VALUE,
			LASTNAME_CLAIM, LASTNAME_CLAIM_VALUE,
			ROLE_CLAIM, ROLE_CLAIM_VALUE,
			ORGANSIATIONSEINHEIT_IDS_CLAIM, ORGE_IDS));
	static final Map<String, Object> CLAIMS_WITHOUT_ORGANSIATIONSEINHEIT_IDS = new HashMap<>(Map.of(
			FIRSTNAME_CLAIM, FIRSTNAME_CLAIM_VALUE,
			LASTNAME_CLAIM, LASTNAME_CLAIM_VALUE,
			ROLE_CLAIM, ROLE_CLAIM_VALUE));

	public static String createToken(String secret, long validity) {
		return createTokenBuilder(secret, validity).compact();
	}

	public static JwtBuilder createTokenBuilder(String secret, long validity) {
		return Jwts.builder()
				.setClaims(CLAIMS)
				.setSubject(SUBJECT)
				.setHeaderParam(TYP, TOKEN_TYPE)
				.setIssuer(TOKEN_ISSUER).setIssuedAt(new Date(System.currentTimeMillis()))
				.setExpiration(new Date(System.currentTimeMillis() + validity))
				.setAudience(TOKEN_AUDIENCE)
				.signWith(SignatureAlgorithm.HS512, secret.getBytes());
	}

}
