/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.user;

import static org.assertj.core.api.Assertions.*;

import java.util.UUID;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.junit.jupiter.params.provider.ValueSource;
import org.mapstruct.factory.Mappers;

class UserIdMapperTest {

	private final UserIdMapper mapper = Mappers.getMapper(UserIdMapper.class);
	private final String userIdValue = UUID.randomUUID().toString();

	@Nested
	class TestFromString {

		@ParameterizedTest
		@NullAndEmptySource
		@ValueSource(strings = {" "})
		void shouldReturnNull(String str) {
			var userId = mapper.fromString(str);

			assertThat(userId).isNull();
		}

		@Test
		void shouldReturnUserId() {
			var userId = mapper.fromString(userIdValue);

			assertThat(userId).isEqualTo(UserId.from(userIdValue));
		}

		@Test
		void shouldTrimUserIdString() {
			var userId = mapper.fromString(" " + userIdValue + " ");

			assertThat(userId).isEqualTo(UserId.from(userIdValue));
		}
	}

	@Nested
	class TestToString {

		@Test
		void shouldReturnUserIdValue() {
			var userIdString = mapper.toString(UserId.from(userIdValue));

			assertThat(userIdString).isEqualTo(userIdValue);
		}
	}
}
